@@ -102,6 +102,7 @@ static void about_cb (GtkWidget *widget, gpointer data);
 static void help_cb(GtkWidget *widget, gpointer data);
 static void fill_in_device_guis(GtkWidget *notebook);
 static void gmix_build_slidernotebook(void);
+static void gmix_hide_last_separator (void);
 
 /*
  * Gnome info:
@@ -328,9 +329,11 @@ main(int argc,
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
-	gnome_init_with_popt_table("gmix", VERSION,
-				   argc, argv, options,
-				   0, NULL);
+	gnome_program_init("gnome-volume-control", VERSION,
+			   LIBGNOMEUI_MODULE, argc, argv,
+			   GNOME_PARAM_POPT_TABLE, options,
+			   GNOME_PARAM_POPT_FLAGS, 0,
+			   GNOME_PARAM_APP_DATADIR, DATADIR, NULL);		
 
 	if (g_file_exists (GNOME_ICONDIR"/gnome-volume.png"))
 		gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-volume.png");
@@ -397,7 +400,7 @@ recsrc_from_channels (device_info *device)
 		channel_info *channel = c->data;
 
 		if (channel->is_record_source == TRUE) {
-			recsrc |= (channel->channel << 1);
+			recsrc |= (1 << channel->channel);
 		}
 	}
 
@@ -1123,37 +1126,37 @@ fill_in_device_guis (GtkWidget *notebook)
 			GtkWidget *label, *mixer, *separator;
 			GtkWidget *hbox;
 			channel_info *ci;
+			char *s;
 
 			ci = c->data;
 
-			hbox = gtk_hbox_new (FALSE, 2);
+			hbox = gtk_hbox_new (FALSE, 3);
 			gtk_widget_show (hbox);
 			
 			gtk_table_attach (GTK_TABLE (table), hbox, i, i+1,
-					  0, 1, GTK_EXPAND | GTK_FILL,
-					  GTK_FILL, 0, 0);
+					  0, 1, 0,
+					  GTK_FILL, 0, 3);
 			
 			if (ci->pixmap) {
 				spixmap = gtk_image_new_from_stock (ci->pixmap,
 								    GTK_ICON_SIZE_BUTTON);
 				gtk_box_pack_start (GTK_BOX (hbox), spixmap, FALSE, FALSE, 0);
-				if (prefs.show_icons == TRUE) {
-					gtk_widget_show (spixmap);
-				}
-				
+				gtk_widget_show (spixmap);
 				ci->icon = spixmap;
 			} else {
 				ci->icon = NULL;
 			}
 			
-			label = gtk_label_new (ci->user_title);
-			gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+			s = g_strdup_printf ("<span size=\"larger\">%s</span>",
+					     ci->user_title);
+			label = gtk_label_new (s);
+			gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+			g_free (s);
+			gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
 			gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
 			gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 			
-			if (prefs.show_labels == TRUE) {
-				gtk_widget_show (label);
-			}
+			gtk_widget_show (label);
 			ci->label = label;
 
 			ci->mixer = make_slider_mixer(ci);
@@ -1251,7 +1254,7 @@ fill_in_device_guis (GtkWidget *notebook)
 			ci->separator = gtk_vseparator_new ();
 			gtk_table_attach (GTK_TABLE (table), ci->separator,
 					  i+1, i+2, 0, 6,
-					  GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+					  GTK_FILL, GTK_EXPAND | GTK_FILL, 2, 0);
 			gtk_widget_show (ci->separator);
 
 			if (ci->visible == FALSE) {
@@ -1281,6 +1284,7 @@ fill_in_device_guis (GtkWidget *notebook)
 		g_list_free (table_focus_chain);
 	}
 
+	gmix_hide_last_separator ();
 }
 
 void
@@ -1675,11 +1679,22 @@ help_cb (GtkWidget *widget,
 {
 	GError *error = NULL;
 	
-	gnome_help_display ("gmix", "gmix", &error);
+	gnome_help_display ("gnome-volume-control", NULL, &error);
 
 	if (error != NULL) {
-		/* FIXME: This is bad :) */
-		g_warning ("%s\n", error->message);
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 ("There was an error displaying help: \n%s"),
+						 error->message);
+
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_widget_show (dialog);
 		g_error_free (error);
 	}
 }
@@ -1747,13 +1762,17 @@ gmix_change_channel (channel_info *ci,
 		}
 		
 		gtk_widget_show (ci->mute);
-		if (ci->icon != NULL) {
+		if (ci->icon != NULL && prefs.show_icons == TRUE) {
 			gtk_widget_show (ci->icon);
 		}
-		gtk_widget_show (ci->label);
+		if (prefs.show_labels == TRUE) {
+			gtk_widget_show (ci->label);
+		}
 	}
 
 	ci->visible = show;
+
+	gmix_hide_last_separator ();
 }
 		
 void
@@ -1812,4 +1831,24 @@ gmix_change_labels (gboolean show)
 	}
 }
 
+static void
+gmix_hide_last_separator (void)
+{
+	GList *d, *c;
+	gboolean hid = FALSE;
+	
+	for (d = devices; d; d = d->next) {
+		device_info *di = d->data;
+
+		for (c = g_list_last (di->channels); c; c = c->prev) {
+			channel_info *ci = c->data;
 		
+			if (ci->visible && hid == FALSE) {
+				gtk_widget_hide (ci->separator);
+				hid = TRUE;
+			} else if (ci->visible) {
+				gtk_widget_show (ci->separator);
+			}
+		}
+	}
+}
